@@ -219,7 +219,7 @@ class numecaParser(OrderedDict):
             self.stringData[0] = "GEOMETRY-TURBO\n"
         self['ROOT'], temp = self.deepTree(self.stringData, 0)
         self['ROOT'] = self.deep_reorder_zrcurve(self['ROOT'])
-        # self['ROOT'] = self.deepReorderNiBladeGeometry(self['ROOT'])
+        self.merge_ZR()
 
     def deep_reorder_zrcurve(self, group):
         new_group = iecGroup()
@@ -488,14 +488,9 @@ class numecaParser(OrderedDict):
     def retrieveNiBladeGeometry(self, row_number=0, blade_number=0):
         row_occurence = -1
         blade_occurence = -1
-        row_key = ""
         blade_key = ""
 
-        for key in self["ROOT"]["GEOMTURBO"].keys():
-            if "nirow" in key or "NIROW" in key:
-                row_occurence += 1
-                if row_occurence == row_number:
-                    row_key = key
+        row_key = self.retrieve_row_key(row_number)
 
         for key in self["ROOT"]["GEOMTURBO"][row_key].keys():
             if "NIBlade" in key or "NIBLADE" in key:
@@ -518,5 +513,105 @@ class numecaParser(OrderedDict):
         return ni_blade_geometry
 
 
+    def set_row_periodicity(self,periodicity,rowNumber=0):
+        row_key = self.retrieve_row_key(rowNumber)
+
+        if (row_key == "" or row_key == None):
+            print("Row not defined")
+        else:
+            self["ROOT"]["GEOMTURBO"][row_key]["PERIODICITY"].value = str(periodicity)
+
+    def get_row_periodicity(self, rowNumber=0):
+        row_key = self.retrieve_row_key(rowNumber)
+        if (row_key == "" or row_key == None):
+            print("Row not defined")
+        else:
+            return(self["ROOT"]["GEOMTURBO"][row_key]["PERIODICITY"].value)
+
+    def retrieve_row_key(self, rowNumber):
+        row_occurence = -1
+        for key in self["ROOT"]["GEOMTURBO"].keys():
+            if "nirow" in key or "NIROW" in key:
+                row_occurence +=1
+                if row_occurence == rowNumber:
+                    return (key)
+        return(None)
+
+    def merge_ZR(self):
+        if "GEOMTURBO" not in self["ROOT"].keys():
+            return
+        basic_curve_dict = self.get_basic_curve_dict()
+        hub_vertex_list = self.get_vertex_list("channel_curve_hub_0")
+        shroud_vertex_list = self.get_vertex_list("channel_curve_shroud_0")
+
+        self.append_and_update_curves(basic_curve_dict,hub_vertex_list)
+        self.append_and_update_curves(basic_curve_dict,shroud_vertex_list)
+
+    def get_vertex_list(self,channel_name):
+        vertex_list = []
+        for key in self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][channel_name].keys():
+            if "VERTEX" in key:
+                vertex_list.append((channel_name,key,self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][channel_name][key].value[0]))
+        return vertex_list
+
+    def get_basic_curve_dict(self):
+        basic_curve_dict = {}
+        for key in self["ROOT"]["GEOMTURBO"]["CHANNEL_0"].keys():
+            if "basic_curve" in key:
+                name = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][key]["NAME"].value
+                zrList = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][key]["zrcurve_0"]
+                basic_curve_dict[name] = (key, zrList)
+        return (basic_curve_dict)
+
+    def append_and_update_curves(self, basic_curve_dict, vertex_list):
+        base_curve = vertex_list[1][2]
+        for curve in vertex_list[2:]:
+            basic_curve_dict[base_curve][1].append(basic_curve_dict[curve[2]][1])
+            key_to_remove = basic_curve_dict[curve[2]][0]
+            del self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][key_to_remove]
+            del self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][curve[0]][curve[1]]
+
+    def exportZRNpyArrays(self):
+        basic_curve_dict = self.get_basic_curve_dict()
+        hub_curve_name = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"]["channel_curve_hub_0"]["VERTEX"].value[0]
+        shroud_curve_name = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"]["channel_curve_shroud_0"]["VERTEX"].value[0]
+        hub_curve = basic_curve_dict[hub_curve_name][1]
+        shroud_curve = basic_curve_dict[shroud_curve_name][1]
+
+        hub_section    = np.vstack([np.zeros(hub_curve.numberOfPoints), hub_curve.R, hub_curve.Z, np.zeros(hub_curve.numberOfPoints)]).transpose()
+        shroud_section = np.vstack([np.zeros(shroud_curve.numberOfPoints), shroud_curve.R, shroud_curve.Z, np.ones(shroud_curve.numberOfPoints)]).transpose()
+        return (np.expand_dims(hub_section,axis=0), np.expand_dims(shroud_section,axis=0))
+
+    def importZRNpyArrays(self, hub_section, shroud_section):
+        self.importZRNpyHubArray(hub_section)
+        self.importZRNpyShroudArray(shroud_section)
+        pass
+
+    def importZRNpyHubArray(self, hub_section):
+        self.importZRNpyGenericArrayWithKey(hub_section,"channel_curve_hub_0")
+        pass
+
+    def importZRNpyShroudArray(self, shroud_section):
+        self.importZRNpyGenericArrayWithKey(shroud_section,"channel_curve_shroud_0")
+        pass
+
+    def importZRNpyGenericArrayWithKey(self, section, key):
+        basic_curve_dict = self.get_basic_curve_dict()
+        curve_name = self["ROOT"]["GEOMTURBO"]["CHANNEL_0"][key]["VERTEX"].value[0]
+        curve = basic_curve_dict[curve_name][1]
+        curve.updateArrays(section[:,2],section[:,1])
+        curve.numberOfPoints = section.shape[0]
+
+    def importZRNpyArray(self, section):
+        if (np.all(section[:,3]==0)):
+            self.importZRNpyHubArray(section)
+        elif (np.all(section[:,3]==1)):
+            self.importZRNpyShroudArray(section)
+        else:
+            print ("ERRORE ARRAY NON COERENTE")
+
+
+
+
 class CustomError(Exception):
     pass
